@@ -21,7 +21,7 @@ def roll_corr(data = None, window=None, min_periods=2, **kwargs):
 
 def compute_rolling_correlation(data = None, feature_name = None, window = 30):
     print('that is where shit is happening')
-    rolling_correlation_df = easy_feature.roll_corr(data[['shifted_close_return',feature_name]], window = window)
+    rolling_correlation_df = roll_corr(data[['shifted_close_return',feature_name]], window = window)
     return rolling_correlation_df
 
 
