@@ -628,7 +628,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         mapping_dictionary = {}
         counter = 1
         for me_signal in self.day_calib_signals_list:
-            print('backtesting '+str(counter)+' over '+str(len(self.day_calib_signals_list)))
+            print(f'backtesting {counter} over {len(self.day_calib_signals_list)}')
             # we have to reload the file each time
             freqly_df = daily_df.copy()
             ## idiosyncratic run itself
@@ -674,7 +674,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         cumulated_signals=cumulated_signals[cumulated_signals.index<=up_to_date]
         backed_up_signal_histo_name =  up_to_date.strftime('%d_%b_%Y')+'_'+self.underlying+'_'+self.backed_up_day_signal_histo_suffix
         print('discretizing the signal')
-        discrete_daily_signals = self.daily_apply_standard_mean_two_states_discretization(signals_df=cumulated_signals, s=self.daily_ia_lookback_window)
+        discrete_daily_signals = self.daily_apply_standard_mean_discretization(signals_df=cumulated_signals, s=self.daily_ia_lookback_window, method ='find_best_three_states_discretization_lo')
         print('discretization done')
         cumulated_signals['trend']=discrete_daily_signals
         saving_path = self.local_root_directory+backed_up_signal_histo_name
@@ -757,7 +757,43 @@ class RealTimeSignalEnsemblingParalleLauncher():
             y_pred_discretized[prediction] = y_next_pred_discrete
         return y_pred_discretized
 
-    def minutely_apply_standard_mean_two_states_discretization(self, signals_df=None, s=21):
+
+    def daily_apply_standard_mean_discretization(self, signals_df=None, s=21, method ='find_best_three_states_discretization_lo'):
+        day_signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        signals_np = signals_df[day_signal_columns].values
+        full_y_true = signals_df['close'].pct_change().fillna(0.)
+        y_pred_discretized = np.zeros(full_y_true.shape)
+        for row_index in range(s, signals_np.shape[0]):
+            starting_train = row_index - s
+            stopping_train = row_index
+            prediction = row_index
+            rolling_np = signals_np[starting_train:stopping_train, :]
+            y_train_true = full_y_true[starting_train:stopping_train]
+            y_train_pred = np.mean(rolling_np, axis=1)
+
+            if method == 'find_best_three_states_discretization':
+                lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization(y_train_pred, y_train_true)
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_three_states(y_next_pred,upper_threshold,lower_threshold)
+            elif method == 'find_best_three_states_discretization_lo':
+                lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization_lo(y_train_pred, y_train_true)
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_three_states_lo(y_next_pred,upper_threshold,lower_threshold)
+            elif method == 'find_best_two_states_discretization' :
+                splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(y_train_pred, y_train_true)
+
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+            elif method == 'find_best_two_states_discretization_lo':
+                splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization_lo(y_train_pred, y_train_true)
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_two_states_lo(y_next_pred,splitting_threshold)
+
+            y_pred_discretized[prediction] = y_next_pred_discrete
+        return y_pred_discretized
+
+
+    def minutely_apply_standard_mean_two_states_discretization(self, signals_df=None, s=21, method ='find_best_two_states_discretization'):
         minute_signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
         signals_np = signals_df[minute_signal_columns].values#[:, len(self.generic_columns):]
         full_y_true = signals_df['close'].pct_change().fillna(0.)
@@ -769,10 +805,24 @@ class RealTimeSignalEnsemblingParalleLauncher():
             rolling_np = signals_np[starting_train:stopping_train, :]
             y_train_true = full_y_true[starting_train:stopping_train]
             y_train_pred = np.mean(rolling_np, axis=1)
-            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
-                y_train_pred, y_train_true)
-            y_next_pred = np.nanmean(signals_np[row_index, :])
-            y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+
+            if method == 'find_best_three_states_discretization':
+                lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization(y_train_pred, y_train_true)
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_three_states(y_next_pred,upper_threshold,lower_threshold)
+            elif method == 'find_best_three_states_discretization_lo':
+                lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization_lo(y_train_pred, y_train_true)
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_three_states_lo(y_next_pred,upper_threshold,lower_threshold)
+            elif method == 'find_best_two_states_discretization' :
+                splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(y_train_pred, y_train_true)
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+            elif method == 'find_best_two_states_discretization_lo':
+                splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization_lo(y_train_pred, y_train_true)
+                y_next_pred = np.nanmean(signals_np[row_index, :])
+                y_next_pred_discrete = signal_utility.apply_discretization_two_states_lo(y_next_pred,splitting_threshold)
+
             y_pred_discretized[prediction] = y_next_pred_discrete
         return y_pred_discretized
 
@@ -1085,8 +1135,8 @@ class RealTimeSignalEnsemblingParalleLauncher():
             counter = 0
             for meArg in self.day_calib_signals_list:
                 triggered = trigger_results[counter]
+                signal_counter = len(self.generic_columns) + counter
                 if triggered:
-                    signal_counter = len(self.generic_columns)+counter
                     for row_idx in range(1, self.day_signal_np.shape[0]):
                         if np.isnan(self.day_signal_np[row_idx, signal_counter]):
                             self.day_signal_np[row_idx, signal_counter] = self.day_signal_np[row_idx-1, signal_counter]
